package com.mortals.xhx.common.utils;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.xhx.base.system.resource.model.ResourceEntity;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.io.FileFilter;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.JarURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.util.*;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.stream.Collectors;

@Slf4j
public class ControllerScanUtil {
    /**
     * 查看类路径
     */
    public static final String[] ULR_VIEW = {"view", "info", "list", "get", "find", "export", "download",
            "index", "bill"};

    public static final String[] WHITE_URL = {"login", "test","file","securitycode"};


    public static void main(String[] args) {
        List<Class<?>> classList = getAllClassByPackageName("com.mortals.xhx");
        //System.out.println(classList); //获取到了所有的类
        getAnnotationInfo(classList);
    }

    public static List<ResourceEntity> getAnnotationInfo(List<Class<?>> clsList) {
        List<ResourceEntity> resourceArrayList = new ArrayList<>();
        if (clsList != null && clsList.size() > 0) {
            for (Class<?> cls : clsList) {

                boolean exits = cls.isAnnotationPresent(RestController.class);
                if (!exits) {
                    continue;
                }
                //白名单地址的 也不需要
                ResourceEntity resourceViewEntity = new ResourceEntity();
                resourceViewEntity.initAttrValue();
                ResourceEntity resourceEditEntity = new ResourceEntity();
                resourceEditEntity.initAttrValue();
                resourceEditEntity.setSourceType(1);

                Method substringMethod = null;
                String result = "";
                try {
                    String packName = cls.getPackage().getName();
                    if (StrUtil.contains(packName, "system")) {
                        //系统管理-xx管理-
                        substringMethod = cls.getMethod("getModuleDesc");
                        result = (String) substringMethod.invoke(cls.newInstance());
                        resourceViewEntity.setName("系统管理-" + result + "管理-查看");
                        resourceEditEntity.setName("系统管理-" + result + "管理-维护");
                    } else {
                        substringMethod = cls.getMethod("getModuleDesc");
                        result = (String) substringMethod.invoke(cls.newInstance());
                        resourceViewEntity.setName(result + "-查看");
                        resourceEditEntity.setName(result + "-维护");
                    }
                } catch (Exception e) {

                }

                RequestMapping requestMappingCls = cls.getAnnotation(RequestMapping.class);
                String prefix = "";
                if (requestMappingCls != null) {
                    prefix = "/" + requestMappingCls.value()[0];
                    boolean b = false;
                    for (String s : WHITE_URL) {
                        if (StrUtil.contains(requestMappingCls.value()[0].toLowerCase(), s)) {
                            b = true;
                            break;
                        }
                    }
                    if (b) {
                        continue;
                    }
                }

                if (ObjectUtils.isEmpty(result)) {
                    if (requestMappingCls != null) {
                        result = StrUtil.removeSuffix(StrUtil.removePrefix(requestMappingCls.value()[0].toUpperCase(), "/"), "/") + "接口模块";
                    }
                    resourceViewEntity.setName(result + "-查看");
                    resourceEditEntity.setName(result + "-维护");
                }
                Set<String> urlSet = new HashSet<>();
                //获取类中的所有的方法
                Method[] methods = cls.getDeclaredMethods();
                if (methods != null && methods.length > 0) {
                    for (Method method : methods) {
                        boolean unAuth = method.isAnnotationPresent(UnAuth.class);
                        if (unAuth) {
                                continue;

                        }

                        boolean mExits = method.isAnnotationPresent(RequestMapping.class);
                        if (mExits) {
                            RequestMapping requestMapping = method.getAnnotation(RequestMapping.class);
                            String url = prefix + "/" + requestMapping.value()[0];
                            url = url.replaceAll("/+", "/");
                            urlSet.add(url);
                        }

                        boolean mExits1 = method.isAnnotationPresent(GetMapping.class);
                        if (mExits1) {
                            GetMapping getMapping = method.getAnnotation(GetMapping.class);
                            String url = prefix + "/" + getMapping.value()[0];
                            url = url.replaceAll("/+", "/");
                            urlSet.add(url);
                        }

                        boolean mExits2 = method.isAnnotationPresent(PostMapping.class);
                        if (mExits2) {
                            PostMapping postMapping = method.getAnnotation(PostMapping.class);
                            String url = prefix + "/" + postMapping.value()[0];
                            url = url.replaceAll("/+", "/");
                            urlSet.add(url);
                        }
                    }
                }

                //获取超类中的方法
                Method[] superMethods = cls.getSuperclass().getDeclaredMethods();
                if (superMethods != null && superMethods.length > 0) {
                    for (Method method : superMethods) {
                        boolean unAuth = method.isAnnotationPresent(UnAuth.class);
                        if (unAuth) {
                            //判断下 是否含有list 如果不包含 则返回
                            boolean mExits2 = method.isAnnotationPresent(PostMapping.class);
                            if (mExits2) {
                                PostMapping postMapping = method.getAnnotation(PostMapping.class);
                                String s =postMapping.value()[0];
                                if(!"list".equalsIgnoreCase(s)){
                                    continue;
                                }
                            }else{
                                continue;
                            }
                        }
                        boolean mExits = method.isAnnotationPresent(RequestMapping.class);
                        if (mExits) {
                            RequestMapping requestMapping = method.getAnnotation(RequestMapping.class);
                            String url = prefix + "/" + requestMapping.value()[0];
                            url = url.replaceAll("/+", "/");
                            urlSet.add(url);
                        }

                        boolean mExits1 = method.isAnnotationPresent(GetMapping.class);
                        if (mExits1) {
                            GetMapping getMapping = method.getAnnotation(GetMapping.class);
                            String url = prefix + "/" + getMapping.value()[0];
                            url = url.replaceAll("/+", "/");
                            urlSet.add(url);
                        }

                        boolean mExits2 = method.isAnnotationPresent(PostMapping.class);
                        if (mExits2) {
                            PostMapping postMapping = method.getAnnotation(PostMapping.class);
                            String url = prefix + "/" + postMapping.value()[0];
                            url = url.replaceAll("/+", "/");
                            urlSet.add(url);
                        }
                    }
                }

                if (!ObjectUtils.isEmpty(urlSet)) {
                    Set<String> UrlViewSet = new HashSet<>();
                    Set<String> UrlEditSet = new HashSet<>();
                    for (String url : urlSet) {
                        String str = StrUtil.subAfter(url, "/", true);
                        boolean bool = false;
                        for (String checkStr : ULR_VIEW) {
                            boolean contains = StrUtil.contains(str.toLowerCase(), checkStr);
                            if (contains) {
                                UrlViewSet.add(url);
                                bool = true;
                                break;
                            }
                        }
                        if (!bool) {
                            UrlEditSet.add(url);
                        }
                    }
                    resourceViewEntity.setUrl(UrlViewSet.stream().collect(Collectors.joining(",")));
                   log.info(JSONObject.toJSONString(resourceViewEntity));
                    resourceArrayList.add(resourceViewEntity);

                    resourceEditEntity.setUrl(UrlEditSet.stream().collect(Collectors.joining(",")));
                    log.info(JSONObject.toJSONString(resourceEditEntity));

                    resourceArrayList.add(resourceEditEntity);
                }
            }
        }

        return resourceArrayList;
    }

    public static List<Class<?>> getAllClassByPackageName(String packageName) {
        // 获取当前包下以及子包下所以的类
        List<Class<?>> returnClassList = getClasses(packageName);
        return returnClassList;
    }

    /**
     * 从包package中获取所有的Class
     *
     * @param packageName
     * @return
     */
    private static List<Class<?>> getClasses(String packageName) {
        // 第一个class类的集合
        List<Class<?>> classes = new ArrayList<Class<?>>();
        // 是否循环迭代
        boolean recursive = true;
        // 获取包的名字 并进行替换
        String packageDirName = packageName.replace('.', '/');
        // 定义一个枚举的集合 并进行循环来处理这个目录下的things
        Enumeration<URL> dirs;
        try {
            dirs = Thread.currentThread().getContextClassLoader().getResources(packageDirName);
            // 循环迭代下去
            while (dirs.hasMoreElements()) {
                // 获取下一个元素
                URL url = dirs.nextElement();
                // 得到协议的名称
                String protocol = url.getProtocol();
                //System.out.println(url.getFile());
                // 如果是以文件的形式保存在服务器上
                if ("file".equals(protocol)) {
                    // 获取包的物理路径
                    String filePath = URLDecoder.decode(url.getFile(), "UTF-8");
                    // 以文件的方式扫描整个包下的文件 并添加到集合中
                    findAndAddClassesInPackageByFile(packageName, filePath, recursive, classes);
                } else if ("jar".equals(protocol)) {
                    // 如果是jar包文件
                    // 定义一个JarFile
                    JarFile jar;
                    try {
                        // 获取jar
                        jar = ((JarURLConnection) url.openConnection()).getJarFile();
                        // 从此jar包 得到一个枚举类
                        Enumeration<JarEntry> entries = jar.entries();
                        // 同样的进行循环迭代
                        while (entries.hasMoreElements()) {
                            // 获取jar里的一个实体 可以是目录 和一些jar包里的其他文件 如META-INF等文件
                            JarEntry entry = entries.nextElement();
                            String name = entry.getName();
                            // 如果是以/开头的
                            if (name.charAt(0) == '/') {
                                // 获取后面的字符串
                                name = name.substring(1);
                            }
                            // 如果前半部分和定义的包名相同
                            if (name.startsWith(packageDirName)) {
                                int idx = name.lastIndexOf('/');
                                // 如果以"/"结尾 是一个包
                                if (idx != -1) {
                                    // 获取包名 把"/"替换成"."
                                    packageName = name.substring(0, idx).replace('/', '.');
                                }
                                // 如果可以迭代下去 并且是一个包
                                if ((idx != -1) || recursive) {
                                    // 如果是一个.class文件 而且不是目录
                                    if (name.endsWith(".class") && !entry.isDirectory()) {
                                        // 去掉后面的".class" 获取真正的类名
                                        String className = name.substring(packageName.length() + 1, name.length() - 6);
                                        try {
                                            // 添加到classes
                                            classes.add(Class.forName(packageName + '.' + className));
                                        } catch (ClassNotFoundException e) {
                                            log.error("异常",e);
                                        }
                                    }
                                }
                            }
                        }
                    } catch (IOException e) {
                        log.error("异常",e);
                    }
                }
            }
        } catch (IOException e) {
            log.error("异常",e);
        }

        return classes;
    }

    /**
     * 以文件的形式来获取包下的所有Class
     *
     * @param packageName
     * @param packagePath
     * @param recursive
     * @param classes
     */
    private static void findAndAddClassesInPackageByFile(String packageName, String packagePath, final boolean recursive, List<Class<?>> classes) {
        // 获取此包的目录 建立一个File
        File dir = new File(packagePath);
        // 如果不存在或者 也不是目录就直接返回
        if (!dir.exists() || !dir.isDirectory()) {
            return;
        }
        // 如果存在 就获取包下的所有文件 包括目录
        File[] dirFiles = dir.listFiles(new FileFilter() {
            // 自定义过滤规则 如果可以循环(包含子目录) 或则是以.class结尾的文件(编译好的java类文件)
            public boolean accept(File file) {
                return (recursive && file.isDirectory()) || (file.getName().endsWith(".class"));
            }
        });

        assert dirFiles != null;
        // 循环所有文件
        for (File file : dirFiles) {
            // 如果是目录 则继续扫描
            if (file.isDirectory()) {
                findAndAddClassesInPackageByFile(packageName + "." + file.getName(), file.getAbsolutePath(), recursive, classes);
            } else {
                // 如果是java类文件 去掉后面的.class 只留下类名
                String className = file.getName().substring(0, file.getName().length() - 6);
                try {
                    // 添加到集合中去
                    classes.add(Class.forName(packageName + '.' + className));
                } catch (ClassNotFoundException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
