package com.mortals.xhx.daemon.task;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.service.ITask;
import com.mortals.framework.service.ITaskExcuteService;
import com.mortals.xhx.base.system.user.service.UserService;
import com.mortals.xhx.common.code.ProductDisEnum;
import com.mortals.xhx.common.code.ProjectStatusEnum;
import com.mortals.xhx.common.code.ProjectTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.pdu.ListItem;
import com.mortals.xhx.common.pdu.NacosResponse;
import com.mortals.xhx.module.setup.model.SetupProjectEntity;
import com.mortals.xhx.module.setup.model.SetupProjectQuery;
import com.mortals.xhx.module.setup.service.SetupProjectService;
import lombok.extern.slf4j.Slf4j;
import org.checkerframework.checker.units.qual.A;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.io.File;
import java.util.*;

/**
 * 检测项目运行状态
 */
@Slf4j
@Service("CheckProjectStatusTask")
public class CheckProjectStatusTaskImpl implements ITaskExcuteService {

    @Autowired
    private SetupProjectService setupProjectService;

    @Override
    public void excuteTask(ITask task) throws AppException {
        log.info("检测项目运行状态任务");
        setupProjectService.updateProjectsStatus();
        log.info("检测项目运行状态任务完毕！");

        //更新项目服务ip

        SetupProjectQuery setupProjectQuery = new SetupProjectQuery();
        setupProjectQuery.setProjectTypeList(new ArrayList<Integer>() {{
            add(ProjectTypeEnum.后端.getValue());
            add(ProjectTypeEnum.PHP后端.getValue());
        }});
        setupProjectQuery.setProjectStatus(ProjectStatusEnum.运行中.getValue());
        setupProjectQuery.setClientToServerIpStatus(YesNoEnum.NO.getValue());

        List<SetupProjectEntity> setupProjectEntities = setupProjectService.find(setupProjectQuery);

        for (SetupProjectEntity setupProjectEntity : setupProjectEntities) {
            if (!ObjectUtils.isEmpty(setupProjectEntity.getClientToServerIp()) && setupProjectEntity.getProjectCode().equals(ProductDisEnum.基础服务后端.getValue())) {
                try {
                    HashMap<String, Object> params = new HashMap<>();
                    params.put("serverIp", setupProjectEntity.getClientToServerIp());
                    //更新基础服务
                    String resp = HttpUtil.get("http://127.0.0.1:11078/base/param/updateServerIp", params);
                    Rest rest = JSON.parseObject(resp, Rest.class);
                    if (YesNoEnum.YES.getValue() == rest.getCode()) {
                        setupProjectEntity.setClientToServerIpStatus(YesNoEnum.YES.getValue());
                        setupProjectService.update(setupProjectEntity);
                    }
                } catch (Exception e) {
                    log.info("更新基础项目服务ip失败");
                }
            }

            if (!ObjectUtils.isEmpty(setupProjectEntity.getClientToServerIp()) && setupProjectEntity.getProjectCode().equals(ProductDisEnum.设备管理服务后端.getValue())) {
                try {
                    HashMap<String, Object> params = new HashMap<>();
                    params.put("serverIp", setupProjectEntity.getClientToServerIp());
                    //更新基础服务
                    String resp = HttpUtil.get("http://127.0.0.1:11078/fm/param/updateServerIp", params);
                    Rest rest = JSON.parseObject(resp, Rest.class);
                    if (YesNoEnum.YES.getValue() == rest.getCode()) {
                        setupProjectEntity.setClientToServerIpStatus(YesNoEnum.YES.getValue());
                        setupProjectService.update(setupProjectEntity);
                    }
                } catch (Exception e) {
                    log.info("更新设备管理项目服务ip失败");
                }
            }

            if (!ObjectUtils.isEmpty(setupProjectEntity.getClientToServerIp()) && setupProjectEntity.getProjectCode().equals(ProductDisEnum.排号系统PHP后端.getValue())) {
                try {
                    HashMap<String, Object> params = new HashMap<>();
                    params.put("clientToServerIp", setupProjectEntity.getClientToServerIp());
                    params.put("innerServerIp", setupProjectEntity.getInnerServerIp());
                    params.put("domain", setupProjectEntity.getDomain());
                    //更新基础服务
                    String resp = HttpUtil.get("http://127.0.0.1:11078/zwfw_api/api/edit_config/setHost", params);
                    Rest rest = JSON.parseObject(resp, Rest.class);
                    if (YesNoEnum.YES.getValue() == rest.getCode()) {
                        setupProjectEntity.setClientToServerIpStatus(YesNoEnum.YES.getValue());
                        setupProjectService.update(setupProjectEntity);
                    }
                } catch (Exception e) {
                    log.info("更新排号项目服务ip失败");
                }
            }

        /*    if (!ObjectUtils.isEmpty(setupProjectEntity.getDomain()) && setupProjectEntity.getProjectCode().equals(ProductDisEnum.排号系统PHP后端.getValue())) {
                try {
                    HashMap<String, Object> params = new HashMap<>();
                    params.put("domain", setupProjectEntity.getDomain());
                    //更新domain服务
                    String resp = HttpUtil.get("http://127.0.0.1:11078/zwfw_api/api/edit_config/setHost", params);
                    Rest rest = JSON.parseObject(resp, Rest.class);
                    if (YesNoEnum.YES.getValue() == rest.getCode()) {
                        setupProjectEntity.setClientToServerIpStatus(YesNoEnum.YES.getValue());
                        setupProjectService.update(setupProjectEntity);
                    }
                } catch (Exception e) {
                    log.info("更新排号项目服务ip失败");
                }
            }*/

        }


    }


    @Override
    public void stopTask(ITask task) throws AppException {

    }


}
