package com.mortals.xhx.module.base.model;

import java.lang.reflect.Type;
import java.util.List;
import java.util.ArrayList;
import com.mortals.xhx.module.base.model.vo.BaseAreaVo;
import com.fasterxml.jackson.annotation.JsonInclude;
import lombok.Data;
import java.io.Serializable;
import java.util.stream.Collectors;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.parser.DefaultJSONParser;
import com.alibaba.fastjson.parser.JSONToken;
import com.alibaba.fastjson.parser.deserializer.ObjectDeserializer;
import org.springframework.util.ObjectUtils;

/**
* 区域前端映射树结构实体类
*
* @author zxfei
* @date 2024-11-01
*/
@Data
public class BaseAreaTreeSelect  implements Serializable {
    private static final long serialVersionUID = 1L;

    /**
    * 节点ID
    */
    private String id;

    /**
     * 节点名称
     */
    private String label;
    /**
     * 区域编码
     */
    private String areaCode;

    /**
     * 是否叶子节点
     */
    private Boolean isLeaf;

    /**
     * 节点类型
     */
    private String type;

    /**
     * 图标
     */
    private String icon;

    /**
     * 区域等级(1.省，2.地市州，3.区县，4.街道，5.社区)
     */
    private Integer areaLevel;

    /**
    * 子节点
    */
    @JsonInclude(JsonInclude.Include.NON_EMPTY)
    private List<BaseAreaTreeSelect> children;

    public BaseAreaTreeSelect() {};

    public BaseAreaTreeSelect(BaseAreaEntity entity) {
        this.id = entity.getIid();
        this.label = entity.getName();
        this.children = new ArrayList();
        if ("False".equalsIgnoreCase(entity.getHaveSonArea())) {
            this.isLeaf = true;

        } else {
            this.isLeaf = false;
        }
        this.areaCode=entity.getAreaCode();
        this.areaLevel=entity.getAreaLevel();
        this.type = "area";
        this.icon = "el-icon-folder";
        if(!ObjectUtils.isEmpty(entity.getChildren())){
            this.children = entity.getChildren().stream().map(BaseAreaTreeSelect::new).collect(Collectors.toList());
        }
    }

    // 反序列化器
    public  class Deserializer implements ObjectDeserializer {
        @Override
        public BaseAreaTreeSelect deserialze(DefaultJSONParser parser, Type type, Object fieldName) {
            BaseAreaTreeSelect node = new BaseAreaTreeSelect();
            JSONObject jsonObject = parser.parseObject();
            node.setId(jsonObject.getString("id"));
            node.setLabel(jsonObject.getString("label"));
            JSONArray jsonArray = jsonObject.getJSONArray("children");
            List<BaseAreaTreeSelect> children = new ArrayList<>();
            if(!ObjectUtils.isEmpty(jsonArray)){
                for (int i = 0; i < jsonArray.size(); i++) {
                    BaseAreaTreeSelect child = JSON.parseObject(jsonArray.getJSONObject(i).toJSONString(), BaseAreaTreeSelect.class);
                    children.add(child);
                }
            }
            node.setChildren(children);
            return node;
        }

        @Override
        public int getFastMatchToken() {
            return JSONToken.LBRACE;
        }
    }

}