package com.mortals.xhx.module.base.service.impl;
import org.springframework.stereotype.Service;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.module.base.dao.BaseAreaDao;
import com.mortals.xhx.module.base.model.BaseAreaEntity;
import com.mortals.xhx.module.base.model.BaseAreaQuery;
import com.mortals.xhx.module.base.service.BaseAreaService;
import com.mortals.xhx.module.base.model.BaseAreaTreeSelect;
import org.springframework.util.ObjectUtils;
import java.util.*;
import java.util.stream.Collectors;
/**
* BaseAreaService
* 区域 service实现
*
* @author zxfei
* @date 2024-11-01
*/
@Service("baseAreaService")
public class BaseAreaServiceImpl extends AbstractCRUDServiceImpl<BaseAreaDao, BaseAreaEntity, Long> implements BaseAreaService {


    /**
    * 修改子元素关系
    *
    * @param baseAreaId       被修改的区域ID
    * @param newAncestors 新的父ID集合
    * @param oldAncestors 旧的父ID集合
    */
    public void updateBaseAreaChildren(Long baseAreaId, String newAncestors, String oldAncestors, Context context) {
        List<BaseAreaEntity> children =  getDao().selectChildrenBaseAreaById(baseAreaId.toString());
        for (BaseAreaEntity child : children) {
            child.setAncestors(child.getAncestors().replace(oldAncestors, newAncestors));
        }
        if (children.size() > 0) {
            this.updateAfter(children, context);
        }
    }


    @Override
    protected void removeBefore(Long[] ids, Context context) throws AppException {
        //有子节点 禁止删除
        if (!ObjectUtils.isEmpty(ids)) {
            BaseAreaEntity baseAreaEntity = this.get(ids[0]);
            if(hasChildByBaseAreaId(baseAreaEntity.getIid())){
                throw new AppException("存在下级区域,不允许删除");
            }

        }
        super.removeBefore(ids, context);
    }

    @Override
    public boolean hasChildByBaseAreaId(String baseAreaId) {
        List<BaseAreaEntity> list = this.find(new BaseAreaQuery().pid(baseAreaId));
        return list.size() > 0 ? true : false;
    }

    @Override
    public List<BaseAreaTreeSelect> buildBaseAreaTreeSelect(List<BaseAreaEntity> list) {
        List<BaseAreaEntity> returnList = new ArrayList<>();
        List<Long> tempList = list.stream().map(BaseAreaEntity::getId).collect(Collectors.toList());

        for (Iterator<BaseAreaEntity> iterator = list.iterator(); iterator.hasNext(); ) {
            BaseAreaEntity baseAreaEntity = iterator.next();
            if (!tempList.contains(baseAreaEntity.getPid())) {
                recursionFn(list, baseAreaEntity);
                returnList.add(baseAreaEntity);
            }
            }
            if (returnList.isEmpty()) {
            returnList = list;
        }
        return returnList.stream().map(BaseAreaTreeSelect::new).collect(Collectors.toList());
    }


    /**
    * 递归列表
    */
    private void recursionFn(List<BaseAreaEntity> list, BaseAreaEntity t) {
        // 得到子节点列表
        List<BaseAreaEntity> childList = getChildList(list, t);
        t.setChildren(childList);
        for (BaseAreaEntity tChild : childList) {
            if (hasChild(list, tChild)) {
                recursionFn(list, tChild);
            }
        }
    }

    /**
    * 判断是否有子节点
    */
    private boolean hasChild(List<BaseAreaEntity> list, BaseAreaEntity t) {
        return getChildList(list, t).size() > 0 ? true : false;
    }

    /**
    * 得到子节点列表
    */
    private List<BaseAreaEntity> getChildList(List<BaseAreaEntity> list, BaseAreaEntity t) {
        return list.stream().map(item->{
            if (!ObjectUtils.isEmpty(item.getPid()) && item.getPid().equals(t.getIid())) {
                return item;
            }
            return null;
            }).filter(f->f!=null).collect(Collectors.toList());
    }

    @Override
    public List<BaseAreaTreeSelect> getListByParentId(String parentId, Context context) {
        if (ObjectUtils.isEmpty(parentId)) {
            parentId = "0";
        }
        //只做一层
        List<BaseAreaTreeSelect> collect = this.find(new BaseAreaQuery().pid(parentId), context).stream().map(item -> new BaseAreaTreeSelect(item)
        ).collect(Collectors.toList());
        return collect;
    }

}