package com.mortals.xhx.module.base.web;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.base.system.param.service.ParamService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.bind.annotation.*;
import com.mortals.framework.model.Context;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.module.base.model.BaseAreaEntity;
import com.mortals.xhx.module.base.service.BaseAreaService;
import com.mortals.xhx.module.base.model.BaseAreaQuery;
import com.mortals.xhx.module.base.model.BaseAreaTreeSelect;
import org.apache.commons.lang3.ArrayUtils;
import com.mortals.framework.util.StringUtils;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import com.alibaba.fastjson.JSONObject;
import java.util.Arrays;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import static com.mortals.framework.ap.SysConstains.*;
import com.mortals.xhx.common.code.*;
/**
*
* 区域
*
* @author zxfei
* @date 2024-11-01
*/
@RestController
@RequestMapping("base/area")
public class BaseAreaController extends BaseCRUDJsonBodyMappingController<BaseAreaService,BaseAreaEntity,Long> {

    @Autowired
    private ParamService paramService;

    public BaseAreaController(){
        super.setModuleDesc( "区域");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
                this.addDict(model, "haveSonArea", HaveSonAreaEnum.getEnumMap());
                this.addDict(model, "haveSonDept", HaveSonDeptEnum.getEnumMap());
                this.addDict(model, "haveGetDept", HaveGetDeptEnum.getEnumMap());
                this.addDict(model, "haveGetMatterList", HaveGetMatterListEnum.getEnumMap());
                this.addDict(model, "areaLevel", AreaLevelEnum.getEnumMap());
                this.addDict(model, "status", StatusEnum.getEnumMap());
        super.init(model, context);
    }


    /**
     * 获取站点下拉树列表
     */
    @PostMapping("treeselect")
    @UnAuth
    public String treeselect() {
        Map<String, Object> model = new HashMap<>();
        JSONObject ret = new JSONObject();
        String busiDesc = "查询" + this.getModuleDesc();
        int code=VALUE_RESULT_SUCCESS;
        try {
            List<BaseAreaEntity> list = this.service.find(new BaseAreaQuery());
            List<BaseAreaTreeSelect> treeSelects = this.service.buildBaseAreaTreeSelect(list);
             model.put("result", treeSelects);
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            this.doException(request, busiDesc, model, e);
        }
        ret.put(KEY_RESULT_DATA, model);
        ret.put(KEY_RESULT_CODE, code);
        return ret.toJSONString();
    }


    /**
    * 根据parentId查询子信息
    */
    @GetMapping(value = "getListByParentId")
    @UnAuth
    public String getListByParentId(String parentId) {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = "查询" + this.getModuleDesc()+"子节点";
        try {
            List<BaseAreaTreeSelect> treeList = this.service.getListByParentId(parentId, getContext());
            model.put(RESULT_KEY,treeList);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
            ret.put(KEY_RESULT_DATA, model);
            recordSysLog(request, busiDesc+"【成功】");
        } catch (Exception e) {
            log.error("根据parentId查询子信息错误", e);
            this.doException(request, busiDesc, model, e);
            ret.put(KEY_RESULT_CODE, VALUE_RESULT_FAILURE);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        return ret.toJSONString();
    }


}