package com.mortals.xhx.module.setup.service.impl;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import cn.hutool.db.DbUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.system.SystemUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.TypeReference;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.service.impl.AbstractCRUDServiceImpl;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.code.ProductDisEnum;
import com.mortals.xhx.common.code.ProjectStatusEnum;
import com.mortals.xhx.common.code.ProjectTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.common.pdu.ListItem;
import com.mortals.xhx.common.pdu.NacosResponse;
import com.mortals.xhx.common.utils.ZipUtils;
import com.mortals.xhx.module.setup.dao.SetupProjectDao;
import com.mortals.xhx.module.setup.mode.DbSetupEntity;
import com.mortals.xhx.module.setup.mode.SiteEntity;
import com.mortals.xhx.module.setup.model.SetupProjectEntity;
import com.mortals.xhx.module.setup.model.SetupProjectQuery;
import com.mortals.xhx.module.setup.service.SetupDbService;
import com.mortals.xhx.module.setup.service.SetupProjectService;
import lombok.extern.slf4j.Slf4j;
import org.codehaus.plexus.archiver.tar.TarGZipUnArchiver;
import org.codehaus.plexus.logging.console.ConsoleLoggerManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.sql.DataSource;
import java.io.*;
import java.nio.charset.Charset;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * SetupProjectService
 * 项目工程信息 service实现
 *
 * @author zxfei
 * @date 2024-10-21
 */
@Service("setupProjectService")
@Slf4j
public class SetupProjectServiceImpl extends AbstractCRUDServiceImpl<SetupProjectDao, SetupProjectEntity, Long> implements SetupProjectService {

    @Autowired
    private UploadService uploadService;

    @Autowired
    private SetupDbService setupDbService;

    @Value("${project.publishPath:'/home/publish'}")
    private String publishPath;

    @Value("${project.nacosUrl}")
    private String nacosUrl;


    @Override
    public Rest<String> distribute(SetupProjectEntity setupProjectEntity, Context context) {
        if (ProjectTypeEnum.后端.getValue() == setupProjectEntity.getProjectType()) {
            //TODO: 后端逻辑
            //TODO: 判断打包文件是否存在，如果不存在 则使用默认的资源文件夹的包文件
            if (!ObjectUtils.isEmpty(setupProjectEntity.getSourceProject())) {
                //判断文件是否存在
                String targetFilePath = uploadService.getFilePath(setupProjectEntity.getSourceProject());
                if (FileUtil.isEmpty(new File(targetFilePath))) {
                    throw new AppException("部署文件不存在！");
                }
                //根据不同的文件类型进行解压
                if (FileUtil.getSuffix(setupProjectEntity.getSourceProject()).equals("zip")) {

                } else if (FileUtil.getSuffix(setupProjectEntity.getSourceProject()).equals("tar.gz")) {

                }
            } else {
                //读取本工程的资源文件 检测压缩包类型
                //String sourcePath = "/project/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getDesc() + "/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getValue() + ".tar.gz";
                //InputStream inputStream = this.getClass().getResourceAsStream(sourcePath);
                String sourcePath = this.publishPath + "/temp/project/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getDesc() + "/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getValue() + ".tar.gz";

                File file = new File(sourcePath);
                log.info("文件路径:{},存在：{}", sourcePath, file.exists());

                File destDir = new File(setupProjectEntity.getProjectPath());

                if (!file.exists()) {
                    throw new AppException("请上传项目工程文件！");
                }
                final TarGZipUnArchiver ua = new TarGZipUnArchiver();
                ConsoleLoggerManager manager = new ConsoleLoggerManager();
                manager.initialize();
                ua.enableLogging(manager.getLoggerForComponent("bla"));
                ua.setSourceFile(file);
                destDir.mkdirs();
                ua.setDestDirectory(destDir);
                ua.extract();
                String publicPath = setupProjectEntity.getProjectPath() + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getValue();
                File sqlPath = new File(publicPath + "/db/base.sql");
                if (sqlPath.exists()) {
                    //导入数据库业务
                    DbSetupEntity dbSetupEntity = new DbSetupEntity();
                    dbSetupEntity.setDbPort(setupProjectEntity.getDbPort());
                    dbSetupEntity.setDbHost(setupProjectEntity.getDbHost());
                    dbSetupEntity.setDbName(setupProjectEntity.getDbName());
                    dbSetupEntity.setUserName(setupProjectEntity.getDbUser());
                    dbSetupEntity.setPassword(setupProjectEntity.getDbPassword());
                    dbSetupEntity.setDbFilePath(sqlPath.getPath());
                    log.info("数据库db路径！{}", dbSetupEntity.getDbFilePath());
                    Rest<String> dbRest = setupDbService.initDb(dbSetupEntity);
                    if (YesNoEnum.YES.getValue() != dbRest.getCode()) {
                        log.info("数据库初始化成功！");
                    }
                }

                setupProjectEntity.setProjectStatus(ProjectStatusEnum.已部署.getValue());
                this.update(setupProjectEntity);

                //判断当前系统类型，执行不同shell 或者 bat脚本
                if (SystemUtil.getOsInfo().isLinux()) {
                    //todo 执行部署脚本
                    try {
                        Runtime.getRuntime().exec("chmod -R 777 " + destDir);
                    } catch (IOException e) {
                        log.error(e.getMessage());
                    }
                    //后端服务，创建service  启动服务
                    callScript("project_deploy.sh", "4", publicPath + "/bin/");
                } else if (SystemUtil.getOsInfo().isWindows()) {
                    //todo
                }
            }

        } else if (ProjectTypeEnum.前端.getValue() == setupProjectEntity.getProjectType()) {
            //TODO: 前端逻辑 静态文件，只需要解压到指定目录即可
            String path = this.publishPath + "/temp/project/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getDesc() + "/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getValue();
            String sourceUiPath = path + ".tar.gz";
            File uiFile = new File(sourceUiPath);

            //查看zip文件是否存在
            String sourceUiZipPath = path + ".zip";
            File uiZipFile = new File(sourceUiZipPath);
            File destDir = new File(setupProjectEntity.getProjectPath()+ setupProjectEntity.getProjectCode());
            if (!destDir.exists()) {
                destDir.mkdir();
            }
            log.info("tar文件存在：{},文件路径:{},zip文件存在：{},文件路径:{}", uiFile.exists(), sourceUiPath, uiZipFile.exists(), sourceUiZipPath);
            if (uiFile.exists()) {
                //解压tar.gz
                final TarGZipUnArchiver ua = new TarGZipUnArchiver();
                ConsoleLoggerManager manager = new ConsoleLoggerManager();
                manager.initialize();
                ua.enableLogging(manager.getLoggerForComponent("bla"));
                ua.setSourceFile(uiFile);
                destDir.mkdirs();
                ua.setDestDirectory(destDir);
                ua.extract();
            } else if (uiZipFile.exists()) {
                ZipUtil.unzip(uiZipFile, destDir);
            } else {
                throw new AppException("请上传项目工程资源文件！");
            }

            setupProjectEntity.setProjectStatus(ProjectStatusEnum.已部署.getValue());
            this.update(setupProjectEntity);

        } else if (ProjectTypeEnum.资源.getValue() == setupProjectEntity.getProjectType()) {
            //TODO: 资源逻辑
        } else if (ProjectTypeEnum.PHP后端.getValue() == setupProjectEntity.getProjectType()) {
            //TODO: 解压php逻辑
            String sourceDirPath = this.publishPath + "/temp/project/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getDesc() + "/";
            String sourcePath = sourceDirPath + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getValue() + ".zip";

            File file = new File(sourcePath);
            log.info("文件路径:{},存在：{}", sourcePath, file.exists());
            File destDir = new File(setupProjectEntity.getProjectPath());
            if (!destDir.exists()) {
                boolean mkdirs = destDir.mkdirs();
                log.info("创建目录：{},{}", destDir.getPath(), mkdirs);
            }
            if (!file.exists()) {
                throw new AppException("请上传项目工程文件！");
            }
            InputStream inputStream = FileUtil.getInputStream(file);
            //获取zip的编码
            try {
                ZipUtil.unzip(inputStream, destDir, Charset.defaultCharset());
            } catch (IllegalArgumentException e) {
                log.error("zip文件编码异常，尝试使用GBK编码,异常：{}", e.getMessage());
                inputStream = FileUtil.getInputStream(file);
                ZipUtil.unzip(inputStream, destDir, Charset.forName("GBK"));
            }

            String publicPath = setupProjectEntity.getProjectPath() + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getValue();
            File sqlPath = new File(publicPath + "/base.sql");
            if (sqlPath.exists()) {
                //导入数据库业务
                DbSetupEntity dbSetupEntity = new DbSetupEntity();
                dbSetupEntity.setDbPort(setupProjectEntity.getDbPort());
                dbSetupEntity.setDbHost(setupProjectEntity.getDbHost());
                dbSetupEntity.setDbName(setupProjectEntity.getDbName());
                dbSetupEntity.setUserName(setupProjectEntity.getDbUser());
                dbSetupEntity.setPassword(setupProjectEntity.getDbPassword());
                dbSetupEntity.setDbFilePath(publicPath + "/base.sql");
                log.info("数据库db路径！{}", dbSetupEntity.getDbFilePath());
                Rest<String> dbRest = setupDbService.initDb(dbSetupEntity);
                if (YesNoEnum.YES.getValue() != dbRest.getCode()) {
                    log.info("数据库初始化成功！");
                }
            } else {
                log.info("数据库文件不存在！{}", sqlPath.getPath());
            }
            setupProjectEntity.setProjectStatus(ProjectStatusEnum.已部署.getValue());
            this.update(setupProjectEntity);

            //判断当前系统类型，执行不同shell 或者 bat脚本
            if (SystemUtil.getOsInfo().isLinux()) {
                //todo 执行部署脚本
                //后端服务，创建service  启动服务
                try {
                    Runtime.getRuntime().exec("chmod -R 777 " + destDir);
                } catch (IOException e) {
                    log.error(e.getMessage());
                }
                callScript("project_deploy.sh", "4", publicPath);
            } else if (SystemUtil.getOsInfo().isWindows()) {
                //todo
            }

        }

        return Rest.ok();
    }

    @Override
    public Rest<String> distributeInit(SetupProjectEntity setupProjectEntity, Context context) {
        //1检测文件是否已经上传
        String projectPath = this.publishPath + "/temp/project/";
        boolean exist = FileUtil.exist(projectPath);
        if (!exist) throw new AppException("请先上传项目工程文件！");

        //2.部署并启动项目  包括网关，基础服务后端，门户服务，设备管理系统，前端，后端资源等等
        buildGateWay(context);
        buildBaseManager(context);
        buildBaseManagerUi(context);
        buildPortalManager(context);
        buildPortalManagerUi(context);
        buildDeviceManager(context);
        buildDeviceManagerUi(context);
        buildLogManager(context);
        buildLogManagerUi(context);
        buildPhp(context);
        buildPhpUi(context);

        return Rest.ok();
    }

    private void buildGateWay(Context context) {
        log.info("开始部署网关服务");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.网关服务.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildBaseManager(Context context) {
        log.info("开始部署基础服务后端");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.基础服务后端.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildBaseManagerUi(Context context) {
        log.info("开始部署基础服务后端前端");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.基础服务前端.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildPortalManager(Context context) {
        log.info("开始部署门户服务");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.门户服务后端.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildPortalManagerUi(Context context) {
        log.info("开始部署门户服务前端");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.门户服务前端.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildDeviceManager(Context context) {
        log.info("开始部署设备管理服务");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.设备管理服务后端.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildDeviceManagerUi(Context context) {
        log.info("开始部署设备管理服务前端");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.设备管理服务前端.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildLogManager(Context context) {
        log.info("开始部署设备管理服务");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.设备管理服务后端.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildLogManagerUi(Context context) {
        log.info("开始部署设备管理服务前端");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.设备管理服务前端.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildPhp(Context context) {
        log.info("开始部署排号php服务");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.排号系统PHP后端.getValue()));
        this.distribute(setupProject, context);
    }

    private void buildPhpUi(Context context) {
        log.info("开始部署排号php服务前端");
        SetupProjectEntity setupProject = this.selectOne(new SetupProjectQuery().projectCode(ProductDisEnum.排号系统管理前端.getValue()));
        this.distribute(setupProject, context);
    }


    @Override
    public Rest<String> appDeploy(SetupProjectEntity setupProjectEntity, Context context) {

        String sourcePath = "./../project/" + ProductDisEnum.getByValue(setupProjectEntity.getProjectCode()).getDesc() + "/app.tar.gz";

        //创建目录
        File appDir = new File(setupProjectEntity.getProjectPath() + "app/");
        File appSiteDir = new File(setupProjectEntity.getProjectPath() + "app/" + setupProjectEntity.getSiteCode() + "/");
        if (!FileUtil.exist(appDir)) {
            FileUtil.mkdir(appDir);
        }

        if (!FileUtil.exist(appSiteDir)) {
            FileUtil.mkdir(appSiteDir);
        }

        File file = new File(sourcePath);
        log.info("文件存在：{}", file.exists());
        InputStream inputStream = FileUtil.getInputStream(file);

        ZipUtils.unGzip(inputStream, appSiteDir.getPath());

        return Rest.ok("应用部署成功！");
    }


    // @Override
    public Rest<String> updateBaseSystem(SiteEntity siteEntity) {

        //获取数据库连接 如果基础服务后端不存在 则提示
        DbSetupEntity dbSetupEntity = new DbSetupEntity();

        dbSetupEntity.setDbName(siteEntity.getDbName());
        dbSetupEntity.setDbHost(siteEntity.getDbHost());
        dbSetupEntity.setDbPort(siteEntity.getDbPort());
        dbSetupEntity.setUserName(siteEntity.getUserName());
        dbSetupEntity.setPassword(siteEntity.getPassword());


        Rest<String> connect = setupDbService.connect(dbSetupEntity);
        if (YesNoEnum.YES.getValue() != connect.getCode()) {
            return Rest.fail("数据库连接失败");
        }

        Rest<DataSource> rest = setupDbService.getDataSource(dbSetupEntity);
        DataSource dataSource = rest.getData();
        Connection connection = null;
        try {
            connection = dataSource.getConnection();
            //updateSite(connection, dbSetupEntity.getSiteEntity()

        } catch (SQLException e) {
            throw new RuntimeException(e.getMessage());
        }


        return null;
    }


    public boolean updateSite(Connection connection, SiteEntity site) throws AppException {
        PreparedStatement statement = null;
        ResultSet resultSet = null;
        try {
            // 构建Sql,参数使用占位符代替
            // 手动拼接SQL
            // String sql = "update mortals_sys_site set ";
            //todo
            String sql = "";
            List<Object> paramList = new ArrayList<Object>();
            //如果某个属性为null，则跳过，不更新此字段
            if (site.getSiteName() != null) {
                sql += "siteName = ?,";
                paramList.add(site.getSiteName());
            }

            // 去除多余的逗号
            sql = sql.substring(0, sql.length() - 1);
            sql += " where id = ?";
            // 添加id到参数列表中
            paramList.add(site.getId());
            // 创建PreparedStatement对象
            statement = connection.prepareStatement(sql);
            // 设置sql中占位符对应的值
            for (int i = 0; i < paramList.size(); i++) {
                statement.setObject(i + 1, paramList.get(i));
            }
            int result = statement.executeUpdate();
            // 如果影响的行数大于0，表示此用户更新成功
            if (result > 0) {
                return true;
            }
        } catch (Exception e) {
            log.error("更新站点信息失败", e);
            throw new AppException(e.getMessage());
        } finally {
            DbUtil.close(resultSet, statement, connection);

        }
        return false;
    }


    private void callScript(String script, String args, String... workspace) {
        try {
            String cmd = "sh " + script + " " + args;
//        	String[] cmd = {"sh", script, "4"};
            File dir = null;
            if (workspace[0] != null) {
                dir = new File(workspace[0]);
                log.info(workspace[0]);
            }
            String[] evnp = {"val=2", "call=Bash Shell"};
            // Process process = Runtime.getRuntime().exec("su");
            Process process = Runtime.getRuntime().exec(cmd, evnp, dir);
//            process = Runtime.getRuntime().exec(cmd);
            BufferedReader input = new BufferedReader(new InputStreamReader(process.getInputStream()));
            String line = "";
            while ((line = input.readLine()) != null) {
                log.info(line);
            }
            input.close();
        } catch (Exception e) {
            log.error("执行脚本失败", e);
        }
    }


    @Override
    public Rest<String> unProjectZip(MultipartFile file, Context context) {

        String originalFilename = file.getOriginalFilename();
        log.info("originalFilename:{}", originalFilename);

        //解压目录，为当前工程目录 /home/publish/setup-manager/project/
        String projectPath = this.publishPath + "/temp/";

        boolean exist = FileUtil.exist(projectPath);
        if (!exist) {
            FileUtil.mkdir(new File(projectPath));
        }
        try {
            if (originalFilename.contains(".zip")) {
                ZipUtil.unzip(file.getInputStream(), new File(projectPath), Charset.defaultCharset());
            } else if (originalFilename.contains(".tar.gz")) {
                ZipUtils.unGzip(file.getInputStream(), projectPath);
            }
        } catch (Exception e) {
            log.error("解压文件失败", e);
            throw new AppException(e.getMessage());

        }
        return Rest.ok();
    }

    @Override
    public Rest<String> updateProjectsStatus() {
        log.info("检测项目状态！");
        List<SetupProjectEntity> setupProjectEntities = this.find(new SetupProjectEntity());
        for (SetupProjectEntity setupProjectEntity : setupProjectEntities) {
            if (ProjectTypeEnum.后端.getValue() == setupProjectEntity.getProjectType()) {
                //todo http://192.168.0.250:8848/nacos/v1/ns/catalog/instances?message=true&serviceName=bill-manager&clusterName=DEFAULT&groupName=DEFAULT_GROUP&pageSize=10&pageNo=1&namespaceId=smart-gov
                String serverUrl = nacosUrl + "/nacos/v1/ns/catalog/instances?message=true&serviceName=" + setupProjectEntity.getProjectCode() + "&clusterName=DEFAULT&groupName=DEFAULT_GROUP&pageSize=10&pageNo=1&namespaceId=smart-gov";
                String resp = "";
                try {
                    resp = HttpUtil.get(serverUrl);
                    NacosResponse nacosResponse = JSON.parseObject(resp, new TypeReference<NacosResponse>() {
                    });
                    int count = nacosResponse.getCount();
                    if (count > 0) {
                        ListItem item = nacosResponse.getList().get(0);
                        boolean healthy = item.isHealthy();
                        if (healthy) {
                            setupProjectEntity.setProjectStatus(ProjectStatusEnum.运行中.getValue());
                        } else {
                            setupProjectEntity.setProjectStatus(ProjectStatusEnum.停止.getValue());
                        }
                        setupProjectEntity.setUpdateTime(new Date());
                        this.update(setupProjectEntity);
                    }

                    //log.info("服务实例详细查询url:{},返回:{},naocsResponse:{}", serverUrl, resp, JSON.toJSONString(nacosResponse));

                } catch (JSONException e) {
                    //log.info("json反序列化异常:{},返回:{}", e.getMessage(), resp);
                    if (setupProjectEntity.getProjectStatus() > ProjectStatusEnum.未部署.getValue()) {
                        setupProjectEntity.setProjectStatus(ProjectStatusEnum.停止.getValue());
                        setupProjectEntity.setUpdateTime(new Date());
                        this.update(setupProjectEntity);
                    }
                } catch (Exception e) {
                   // log.error("发生异常:{}", e.getMessage());
                }
            } else if (ProjectTypeEnum.前端.getValue() == setupProjectEntity.getProjectType()) {
                //todo 前端 判断路径文件是否存在 如果存在 则代表运行
                String projectPath = setupProjectEntity.getProjectPath();
                String projectCode = setupProjectEntity.getProjectCode();
                String destPath = projectPath + File.separator + projectCode + File.separator + "dist" + File.separator + "index.html";
                File uiFile = new File(destPath);
                if (uiFile.exists()) {
                    setupProjectEntity.setProjectStatus(ProjectStatusEnum.运行中.getValue());
                    setupProjectEntity.setUpdateTime(new Date());
                    this.update(setupProjectEntity);
                } else {
                    setupProjectEntity.setProjectStatus(ProjectStatusEnum.未部署.getValue());
                    setupProjectEntity.setUpdateTime(new Date());
                    this.update(setupProjectEntity);
                }
               // log.info("前端资源路径:{},是否存在:{}", destPath, uiFile.exists());
            } else if (ProjectTypeEnum.PHP后端.getValue() == setupProjectEntity.getProjectType()) {
                //todo 前端 判断路径文件是否存在 如果存在 则代表运行
                if (ProductDisEnum.排号系统PHP后端.getValue().equals(setupProjectEntity.getProjectCode())) {
                    String serverUrl = "http://127.0.0.1:11078/zwfw_api/api/upload/dictionary";
                    String resp = "";
                    try {
                        resp = HttpUtil.get(serverUrl);
                        JSON.parseObject(resp);
                        setupProjectEntity.setProjectStatus(ProjectStatusEnum.运行中.getValue());
                        setupProjectEntity.setUpdateTime(new Date());
                        this.update(setupProjectEntity);
                    } catch (JSONException e) {
                       // log.info("json反序列化异常:{},返回:{}", e.getMessage(), resp);
                        if (setupProjectEntity.getProjectStatus() > ProjectStatusEnum.未部署.getValue()) {
                            setupProjectEntity.setProjectStatus(ProjectStatusEnum.停止.getValue());
                            setupProjectEntity.setUpdateTime(new Date());
                            this.update(setupProjectEntity);
                        }
                    } catch (Exception e) {
                       // log.error("发生异常:{}", e.getMessage());
                    }
                }

                if (ProductDisEnum.窗口服务行为监察PHP后端.getValue().equals(setupProjectEntity.getProjectCode())) {
                    String serverUrl = "http://127.0.0.1:11030/admin/waited/verify";
                    String resp = "";
                    try {
                        resp = HttpUtil.get(serverUrl);
                        JSON.parseObject(resp);
                        setupProjectEntity.setProjectStatus(ProjectStatusEnum.运行中.getValue());
                        setupProjectEntity.setUpdateTime(new Date());
                        this.update(setupProjectEntity);
                    } catch (JSONException e) {
                        //log.info("json反序列化异常:{},返回:{}", e.getMessage(), resp);
                        if (setupProjectEntity.getProjectStatus() > ProjectStatusEnum.未部署.getValue()) {
                            setupProjectEntity.setProjectStatus(ProjectStatusEnum.停止.getValue());
                            setupProjectEntity.setUpdateTime(new Date());
                            this.update(setupProjectEntity);
                        }
                    } catch (Exception e) {
                       // log.error("发生异常:{}", e.getMessage());
                    }
                }
            } else {

            }
        }

        return Rest.ok();
    }


    public static void main(String[] args) {
        System.out.println(FileUtil.getSuffix("test.tar.gz"));

        File file = FileUtil.file("F:/test.tar.gz");


        InputStream inputStream = FileUtil.getInputStream(file);

        ///Hutoo.unGzip(inputStream, setupProjectEntity.getProjectPath());

        //Hutool

        ZipUtils.unGzip(inputStream, "F:/");


        // CompressorInputStream gz = CompressUtil.getIn("gz", FileUtil.getInputStream(file));

/*        Extractor extractor = 	CompressUtil.createExtractor(
                CharsetUtil.defaultCharset(),
                FileUtil.file("F:/test.tar.gz"));

        extractor.extract(FileUtil.file("F:/"));*/

        //CallShell call = new CallShell();
        //call.callScript("test.sh", "4", "/root/experiment/");
    }
}