package com.mortals.xhx.module.setup.web;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.setup.mode.DbSetupEntity;
import com.mortals.xhx.module.setup.service.SetupDbService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import static com.mortals.framework.web.BaseController.*;


@RestController
@RequestMapping("db")
@Slf4j
public class SetupDbController {

    @Autowired
    private SetupDbService setupDbService;

    @PostMapping("/connect")
    @UnAuth
    public String connect(@RequestBody DbSetupEntity dbSetupEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (ObjectUtils.isEmpty(dbSetupEntity.getDbName())) {
                throw new AppException("请选择产品数据库名称");
            }

            Rest<String> rest = setupDbService.connect(dbSetupEntity);
            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                ret.put(KEY_RESULT_MSG, "连接数据库成功!");
            } else {
                throw new AppException(rest.getMsg());
            }
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("链接数据库异常", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);
        return ret.toJSONString();
    }


    @PostMapping("/init")
    @UnAuth
    public String initDb(@RequestBody DbSetupEntity dbSetupEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (ObjectUtils.isEmpty(dbSetupEntity.getDbFilePath())) {
                throw new AppException("请上传sql文件");
            }
            if (ObjectUtils.isEmpty(dbSetupEntity.getDbName())) {
                throw new AppException("请输入数据库名称");
            }
            Rest<String> rest = setupDbService.initDb(dbSetupEntity);
            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                ret.put(KEY_RESULT_MSG, "初始化数据库成功!");
            } else {
                throw new AppException(rest.getMsg());
            }
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("导入数据库失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);
        return ret.toJSONString();
    }

}