package com.mortals.xhx.module.setup.web;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.lang.Validator;
import cn.hutool.core.net.Ipv4Util;
import cn.hutool.http.HttpUtil;
import cn.hutool.system.SystemUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.TypeReference;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.common.Rest;
import com.mortals.framework.exception.AppException;
import com.mortals.framework.model.Context;
import com.mortals.framework.model.OrderCol;
import com.mortals.framework.web.BaseCRUDJsonBodyMappingController;
import com.mortals.xhx.common.code.ProductDisEnum;
import com.mortals.xhx.common.code.ProjectStatusEnum;
import com.mortals.xhx.common.code.ProjectTypeEnum;
import com.mortals.xhx.common.code.YesNoEnum;
import com.mortals.xhx.module.setup.mode.DbSetupEntity;
import com.mortals.xhx.module.setup.mode.ProjectSetupEntity;
import com.mortals.xhx.module.setup.mode.SiteEntity;
import com.mortals.xhx.module.setup.model.SetupProjectEntity;
import com.mortals.xhx.module.setup.model.SetupProjectQuery;
import com.mortals.xhx.module.setup.service.SetupDbService;
import com.mortals.xhx.module.setup.service.SetupProjectService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.codehaus.plexus.archiver.tar.TarGZipUnArchiver;
import org.codehaus.plexus.logging.console.ConsoleLoggerManager;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.mortals.xhx.common.code.ProductDisEnum.*;

/**
 * 项目工程信息
 *
 * @author zxfei
 * @date 2024-10-21
 */
@RestController
@RequestMapping("setup/project")
@Slf4j
public class SetupProjectController extends BaseCRUDJsonBodyMappingController<SetupProjectService, SetupProjectEntity, Long> {


    @Value("${project.nacosUrl}")
    private String nacosUrl;

    @Autowired
    private SetupDbService setupDbService;
    @Value("${project.publishPath:'/home/publish'}")
    private String publishPath;


    public SetupProjectController() {
        super.setModuleDesc("项目工程信息");
    }

    @Override
    protected void init(Map<String, Object> model, Context context) {
        this.addDict(model, "projectType", ProjectTypeEnum.getEnumMap());
        this.addDict(model, "projectStatus", ProjectStatusEnum.getEnumMap());
        super.init(model, context);
    }

    @Override
    protected void doListBefore(SetupProjectEntity query, Map<String, Object> model, Context context) throws AppException {
        query.setSize(50);
        // query.setOrderColList(Arrays.asList(new OrderCol("projectCode"),new OrderCol("orderNum")));
        query.setOrderColList(Arrays.asList(new OrderCol("orderNum")));
        super.doListBefore(query, model, context);
    }


    @Override
    protected void doListAfter(SetupProjectEntity query, List<SetupProjectEntity> list, Context context) throws AppException {
        //todo 更新项目状态 通过shell cmd 命令等 查询项目状态

        super.doListAfter(query, list, context);
    }

    /**
     * 部署
     */
    @ApiOperation(value = "项目部署", notes = "部署")
    @PostMapping(value = "distribute")
    public Rest<Void> distribute(@RequestBody SetupProjectEntity setupProjectEntity) {
        String busiDesc = this.getModuleDesc() + "设备模块部署";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            this.service.distribute(setupProjectEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("项目部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }


    /**
     * 批量部署
     */
    @ApiOperation(value = "批量项目部署", notes = "部署")
    @PostMapping(value = "batchDistribute")
    public Rest<Void> batchDistribute(@RequestBody SetupProjectEntity setupProjectEntity) {
        String busiDesc = this.getModuleDesc() + "设备模块部署";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            if (!ObjectUtils.isEmpty(setupProjectEntity.getIdList())) {
                SetupProjectQuery setupProjectQuery = new SetupProjectQuery();
                setupProjectQuery.setIdList(setupProjectEntity.getIdList());
                List<SetupProjectEntity> setupProjectEntities = this.service.find(setupProjectQuery, getContext());
                for (SetupProjectEntity projectEntity : setupProjectEntities) {
                    this.service.distribute(projectEntity, getContext());
                }
            }
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("批量项目部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }


    /**
     * 部署
     */
    @ApiOperation(value = "项目初始部署", notes = "部署")
    @PostMapping(value = "init")
    public Rest<Void> init(@RequestBody SetupProjectEntity setupProjectEntity) {
        String busiDesc = this.getModuleDesc() + "设备模块部署";
        Rest<Void> rest = Rest.ok(busiDesc + " 【成功】");
        try {
            this.service.distributeInit(setupProjectEntity, getContext());
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            log.error("项目部署", e);
            rest = Rest.fail(super.convertException(e));
        }
        return rest;
    }


    @PostMapping("/dbConnect")
    @UnAuth
    public String connect(@RequestBody SetupProjectEntity setupProjectEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (ObjectUtils.isEmpty(setupProjectEntity.getDbName())) {
                throw new AppException("请选择产品数据库名称");
            }

            //导入数据库业务
            DbSetupEntity dbSetupEntity = new DbSetupEntity();
            dbSetupEntity.setDbPort(setupProjectEntity.getDbPort());
            dbSetupEntity.setDbHost(setupProjectEntity.getDbHost());
            dbSetupEntity.setDbName(setupProjectEntity.getDbName());
            dbSetupEntity.setUserName(setupProjectEntity.getDbUser());
            dbSetupEntity.setPassword(setupProjectEntity.getDbPassword());

            Rest<String> rest = setupDbService.connect(dbSetupEntity);
            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                ret.put(KEY_RESULT_MSG, "连接数据库成功!");
            } else {
                throw new AppException(rest.getMsg());
            }
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("链接数据库异常", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);
        return ret.toJSONString();
    }


    /**
     * 更新终端访问服务端ip地址
     *
     * @return
     */
    @PostMapping("/serverIp/update")
    @UnAuth
    public String serverIpUpdate(@RequestBody SetupProjectEntity setupProjectEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;

        String clientToServerIp = setupProjectEntity.getClientToServerIp();
        String innerServerIp = setupProjectEntity.getInnerServerIp();
        String domain = setupProjectEntity.getDomain();

        try {
            boolean ipv4 = Validator.isIpv4(clientToServerIp);
            if (!ipv4) {
                throw new AppException("请输入正确的ip地址");
            }

            SetupProjectEntity setupProjectBase = this.service.selectOne(new SetupProjectQuery().projectCode(基础服务后端.getValue()));
            if (!ObjectUtils.isEmpty(setupProjectBase)) {
                setupProjectBase.setClientToServerIp(clientToServerIp);
                setupProjectBase.setClientToServerIpStatus(YesNoEnum.NO.getValue());

                setupProjectBase.setInnerServerIp(innerServerIp);
                setupProjectBase.setInnerServerIpStatus(YesNoEnum.NO.getValue());

                this.service.update(setupProjectBase);
            }

            SetupProjectEntity setupProjectDevice = this.service.selectOne(new SetupProjectQuery().projectCode(设备管理服务后端.getValue()));
            if (!ObjectUtils.isEmpty(setupProjectDevice)) {
                setupProjectDevice.setClientToServerIp(clientToServerIp);
                setupProjectDevice.setClientToServerIpStatus(YesNoEnum.NO.getValue());

                setupProjectDevice.setInnerServerIp(innerServerIp);
                setupProjectDevice.setInnerServerIpStatus(YesNoEnum.NO.getValue());
                this.service.update(setupProjectDevice);
            }


            SetupProjectEntity setupProjectPHP = this.service.selectOne(new SetupProjectQuery().projectCode(排号系统PHP后端.getValue()));
            if (!ObjectUtils.isEmpty(setupProjectPHP)) {
                setupProjectPHP.setClientToServerIp(clientToServerIp);
                setupProjectPHP.setClientToServerIpStatus(YesNoEnum.NO.getValue());

                setupProjectPHP.setInnerServerIp(innerServerIp);
                setupProjectPHP.setInnerServerIpStatus(YesNoEnum.NO.getValue());

                setupProjectPHP.setDomain(domain);
                setupProjectPHP.setDomainStatus(YesNoEnum.NO.getValue());
                this.service.update(setupProjectPHP);
            }

/*
            ipv4 = Validator.isIpv4(innerServerIp);
            if (!ipv4) {
                throw new AppException("请输入正确的ip地址");
            }*/
/*

            HashMap<String, Object> params = new HashMap<>();
            params.put("serverIp", clientToServerIp);
            //更新基础服务
            String resp = HttpUtil.get("http://127.0.0.1:11078/base/param/updateServerIp", params);
            Rest rest = JSON.parseObject(resp, Rest.class);
            if (YesNoEnum.NO.getValue() == rest.getCode()) {
                throw new AppException(rest.getMsg());
            }
            //更新设备管理系统
            resp = HttpUtil.get("http://127.0.0.1:11078/fm/param/updateServerIp", params);
            rest = JSON.parseObject(resp, Rest.class);
            if (YesNoEnum.NO.getValue() == rest.getCode()) {
                throw new AppException(rest.getMsg());
            }
*/

            //更新php 终端访问地址 todo

        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("服务ip更新失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
            return ret.toJSONString();
        }
        ret.put(KEY_RESULT_CODE, code);
        ret.put(KEY_RESULT_MSG, "服务ip更新成功");

        return ret.toJSONString();
    }


    /**
     * 更新终端访问服务端ip地址
     *
     * @return
     */
    @GetMapping("/serverIp/view")
    public String serverIpUpdate() {
        JSONObject ret = new JSONObject();
        SetupProjectEntity setupProjectEntity = new SetupProjectEntity();
        setupProjectEntity.initAttrValue();

        SetupProjectEntity setupProjectBase = this.service.selectOne(new SetupProjectQuery().projectCode(基础服务后端.getValue()));
        if (!ObjectUtils.isEmpty(setupProjectBase)) {
            setupProjectEntity.setClientToServerIp(setupProjectBase.getClientToServerIp());
            setupProjectEntity.setInnerServerIp(setupProjectBase.getInnerServerIp());
        }
        SetupProjectEntity setupProjectPHP = this.service.selectOne(new SetupProjectQuery().projectCode(排号系统PHP后端.getValue()));
        if (!ObjectUtils.isEmpty(setupProjectPHP)) {
            setupProjectEntity.setDomain(setupProjectPHP.getDomain());
        }

        ret.put(KEY_RESULT_DATA, setupProjectEntity);

        ret.put(KEY_RESULT_CODE, VALUE_RESULT_SUCCESS);
        ret.put(KEY_RESULT_MSG, "服务ip查看成功！");

        return ret.toJSONString();
    }


    /**
     * 更新基础服务子区域
     *
     * @return
     */
    @PostMapping("/area/update")
    @UnAuth
    public String areaUpdate(@RequestBody SiteEntity siteEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;

        if (ObjectUtils.isEmpty(siteEntity.getAreaCode())) throw new AppException("请输入区域编码");

        try {
            HashMap<String, String> params = new HashMap<>();
            params.put("areaCode", siteEntity.getAreaCode());

            String resp = HttpUtil.post("http://127.0.0.1:11078/base/area/genSubAreaByAreaName", JSON.toJSONString(params));
            Rest rest = JSON.parseObject(resp, Rest.class);

            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                ret.put(KEY_RESULT_MSG, "子区域更新成功");
                SetupProjectEntity setupProjectEntity = new SetupProjectEntity();
                setupProjectEntity.setId(siteEntity.getId());
                setupProjectEntity.setAreaCode(siteEntity.getAreaCode());
                this.service.update(setupProjectEntity);
            }

            ret.put(KEY_RESULT_MSG, "子区域更新失败");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("子区域更新失败失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }


    /**
     * 站点事项数据更新
     *
     * @return
     */
    @PostMapping("/matter/update")
    public String matterUpdate(@RequestBody SiteEntity siteEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;

        SetupProjectEntity setupProject = this.service.get(siteEntity.getId());
        if (ObjectUtils.isEmpty(setupProject)) throw new AppException("未找到项目信息");
        if (ObjectUtils.isEmpty(setupProject.getSiteId())) throw new AppException("请同步更新站点信息!");

        //基础服务必须运行中
        SetupProjectQuery query = new SetupProjectQuery();
        query.setProjectCodeList(Arrays.asList(基础服务后端.getValue(), 网关服务.getValue()));
        List<SetupProjectEntity> setupProjectEntities = this.service.find(query);
        for (SetupProjectEntity setupProjectEntity : setupProjectEntities) {
            if (ProjectStatusEnum.运行中.getValue() != setupProjectEntity.getProjectStatus()) {
                throw new AppException(String.format("%s服务状态未运行，请等待服务运行后再进行事项数据同步!", setupProjectEntity.getName()));
            }
        }

        try {
            HashMap<String, Object> params = new HashMap<>();
            params.put("id", setupProject.getSiteId());

            String resp = HttpUtil.post("http://127.0.0.1:11078/base/site/syncGovMatterBySiteId", JSON.toJSONString(params));
            Rest rest = JSON.parseObject(resp, Rest.class);

            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                ret.put(KEY_RESULT_MSG, "更新事项命令下发成功!");
                this.service.update(setupProject);
            } else {
                ret.put(KEY_RESULT_MSG, "更新事项命令下发失败");
            }

        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("更新事项命令下发失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }


    /**
     * 创建站点
     *
     * @return
     */
    @PostMapping("/site/updateSite")
    @UnAuth
    public String updateSite(@RequestBody SiteEntity siteEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        if (ObjectUtils.isEmpty(siteEntity.getSiteName())) throw new AppException("请输入站点名称!");
        if (ObjectUtils.isEmpty(siteEntity.getAreaCode())) throw new AppException("请先选择区域!");

        //基础服务与门户服务必须运行中
        SetupProjectQuery query = new SetupProjectQuery();
        query.setProjectCodeList(Arrays.asList(基础服务后端.getValue(), 门户服务后端.getValue(), 网关服务.getValue()));
        List<SetupProjectEntity> setupProjectEntities = this.service.find(query);
        for (SetupProjectEntity setupProjectEntity : setupProjectEntities) {
            if (ProjectStatusEnum.运行中.getValue() != setupProjectEntity.getProjectStatus()) {
                throw new AppException(String.format("%s服务状态未运行，请等待服务运行后再进行站点创建!", setupProjectEntity.getName()));
            }
        }

        try {
            //生成子区域/base/area/genSubAreaByAreaName
            String resp = HttpUtil.post("http://127.0.0.1:11078/base/base/area/genSubAreaByAreaName", JSON.toJSONString(siteEntity));
            Rest<String> rest = JSON.parseObject(resp, new TypeReference<Rest<String>>() {
            });
            if (YesNoEnum.YES.getValue() == rest.getCode()) {
                resp = HttpUtil.post("http://127.0.0.1:11078/base/site/api/add", JSON.toJSONString(siteEntity));
                log.info("站点添加返回:{}", resp);
                Rest<SiteEntity> siteAddRest = JSON.parseObject(resp, new TypeReference<Rest<SiteEntity>>() {
                });

                if (YesNoEnum.YES.getValue() == siteAddRest.getCode()) {
                    //更新所有项目的站点编码与名称
                    this.service.find(new SetupProjectQuery()).forEach(setupProjectEntity -> {
                        setupProjectEntity.setSiteName(siteAddRest.getData().getSiteName());
                        setupProjectEntity.setSiteCode(siteAddRest.getData().getSiteCode());
                        setupProjectEntity.setSiteId(siteAddRest.getData().getId());
                        setupProjectEntity.setAreaCode(siteAddRest.getData().getAreaCode());
                        setupProjectEntity.setAreaName(siteAddRest.getData().getAreaName());
                        this.service.update(setupProjectEntity);
                    });

                    //todo 在/home/publish/app/ 目录下 更加站点编码生成基础服务应用 创建自助服务应用
                    String projectPath = this.publishPath + "/app/" + siteAddRest.getData().getSiteCode();
                    //解压资源文件
                    String sourcePath = this.publishPath + "/temp/project/" + 基础服务后端.getValue() + "/app.tar.gz";
                    File file = new File(sourcePath);
                    log.info("文件存在：{} ,文件地址:{}", file.exists(), sourcePath);
                    File destDir = new File(projectPath);
                    if (!file.exists()) {
                        throw new AppException("请上传项目工程文件！");
                    }
                    final TarGZipUnArchiver ua = new TarGZipUnArchiver();
                    ConsoleLoggerManager manager = new ConsoleLoggerManager();
                    manager.initialize();
                    ua.enableLogging(manager.getLoggerForComponent("bla"));
                    ua.setSourceFile(file);
                    destDir.mkdirs();
                    ua.setDestDirectory(destDir);
                    ua.extract();

                    projectPath = this.publishPath + "/app/";
                    //解压资源文件
                    sourcePath = this.publishPath + "/temp/project/" + 基础服务后端.getValue() + "/other.tar.gz";
                    file = new File(sourcePath);
                    destDir = new File(projectPath);
                    manager.initialize();
                    ua.enableLogging(manager.getLoggerForComponent("bla"));
                    ua.setSourceFile(file);
                    destDir.mkdirs();
                    ua.setDestDirectory(destDir);
                    ua.extract();

                    //todo 更新门户站点信息，用户站点树等 user/initData
                    JSONObject json = new JSONObject();
                    //1,3,4,5,9,10,11,12,13,14,15,16,17,19,20,21,22,23,24,25,26,28
                    json.put("siteIds", siteAddRest.getData().getId());
                    //511500000000
                    json.put("areaCodes", siteAddRest.getData().getAreaCode());
                    //[{"areaCode":"511500000000","label":"宜宾市"}]
                    JSONArray jsonArray = new JSONArray();
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("areaCode", siteAddRest.getData().getAreaCode());
                    jsonObject.put("label", siteAddRest.getData().getAreaName());
                    jsonArray.add(jsonObject);
                    json.put("areaNames", jsonArray.toJSONString());
                    String portalResp = HttpUtil.post("http://127.0.0.1:11078/zwfw/user/initData", json.toJSONString());
                    log.info("门户站点更新返回:{}", portalResp);
                } else {
                    throw new AppException("站点添加失败! " + rest.getMsg());
                }
            } else {
                throw new AppException("子区域更新失败! " + rest.getMsg());
            }

            ret.put(KEY_RESULT_MSG, "站点添加成功！");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("站点添加失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }


    /**
     * 自助服务应用部署
     *
     * @return
     */
    @PostMapping("/app/deploy")
    @UnAuth
    public String appDeploy(@RequestBody SiteEntity siteEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;

        if (ObjectUtils.isEmpty(siteEntity.getSiteCode())) throw new AppException("请先创建站点后再部署自助服务应用！");

        try {
            //todo 在/home/publish/app/ 目录下 创建自助服务应用
            String projectPath = this.publishPath + "/app/" + siteEntity.getSiteCode();
            boolean exist = FileUtil.exist(projectPath);
            if (!exist) {
                FileUtil.mkdir(projectPath);
            }
            //解压

            ret.put(KEY_RESULT_MSG, "子区域更新失败");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("查询服务状态失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }


    /**
     * 服务列表查询
     *
     * @return
     */
    @PostMapping("/instances/list")
    @UnAuth
    public String listInstances() {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        try {
            // curl -X GET 'http://127.0.0.1:8848/nacos/v1/ns/catalog/services?pageNo=1&pageSize=100&namespaceId=smart-gov'
            String resp = HttpUtil.get(nacosUrl + "/v1/ns/catalog/services?pageNo=1&pageSize=100&namespaceId=smart-gov");
            log.info("服务列表查询返回:{}", resp);
            ret.put(KEY_RESULT_DATA, resp);
            ret.put(KEY_RESULT_MSG, "列表查询成功");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("列表查询失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);

        return ret.toJSONString();
    }

    /**
     * 服务实例详细查询
     *
     * @param projectSetupEntity
     * @return
     */
    @PostMapping("/instances/get")
    @UnAuth
    public String getInstances(@RequestBody ProjectSetupEntity projectSetupEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;

        SetupProjectEntity projectEntity = this.service.get(projectSetupEntity.getId());
        if (ObjectUtils.isEmpty(projectEntity)) throw new AppException("项目不存在");

        try {
            // curl -X GET 'http://127.0.0.1:8848/nacos/v1/ns/catalog/instances?serviceName=base-manager&clusterName=DEFAULT&groupName=DEFAULT_GROUP&pageSize=10&pageNo=1&namespaceId=smart-gov'
            String resp = HttpUtil.get(nacosUrl + "/v1/ns/catalog/instances?serviceName=" + projectEntity.getProjectCode() + "&clusterName=DEFAULT&groupName=DEFAULT_GROUP&pageSize=10&pageNo=1&namespaceId=smart-gov");
            log.info("服务实例详细查询返回:{}", resp);
            ret.put(KEY_RESULT_DATA, resp);
            ret.put(KEY_RESULT_MSG, "服务实例详细查询成功");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("服务实例详细查询失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);
        return ret.toJSONString();
    }


    @PostMapping("/importProjectData")
    @UnAuth
    public String importDoc(MultipartFile file) {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        Context context = getContext();
        String busiDesc = "导入" + getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            //解压
            this.service.unProjectZip(file, context);
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            doException(request, busiDesc, model, e);
        }
        ret.put(KEY_RESULT_CODE, code);
        ret.put(KEY_RESULT_MSG, "导入成功！");
        return ret.toJSONString();
    }

    @PostMapping("/importProjectDataInit")
    @UnAuth
    public String importProjectDataInit(MultipartFile file) {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        Context context = getContext();
        String busiDesc = "导入" + getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            //解压
            this.service.unProjectZip(file, context);
          /*  //初始化项目
            SetupProjectEntity setupProjectEntity = new SetupProjectEntity();
            this.service.distributeInit(setupProjectEntity, getContext());*/
            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            doException(request, busiDesc, model, e);
        }
        ret.put(KEY_RESULT_CODE, code);
        ret.put(KEY_RESULT_MSG, "导入成功！");
        return ret.toJSONString();
    }

    @PostMapping("/projectDataInit")
    @UnAuth
    public String projectDataInit() {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = "项目初始化" + getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            //初始化项目
            SetupProjectEntity setupProjectEntity = new SetupProjectEntity();
            this.service.distributeInit(setupProjectEntity, getContext());

            this.service.updateProjectsStatus();

            recordSysLog(request, busiDesc + " 【成功】");
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            doException(request, busiDesc, model, e);
        }
        ret.put(KEY_RESULT_CODE, code);
        ret.put(KEY_RESULT_MSG, "项目初始化成功！");
        return ret.toJSONString();
    }


    @PostMapping("/projectStatusUpdate")
    @UnAuth
    public String projectStatusUpdate() {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = "项目运行状态更新" + getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            this.service.updateProjectsStatus();
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            doException(request, busiDesc, model, e);
        }
        ret.put(KEY_RESULT_CODE, code);
        ret.put(KEY_RESULT_MSG, "项目运行状态更新成功！");
        return ret.toJSONString();
    }


    @PostMapping("/start")
    @UnAuth
    public String start(@RequestBody SetupProjectEntity setupProject) {
        JSONObject ret = new JSONObject();
        Map<String, Object> model = new HashMap<>();
        String busiDesc = "项目运行启动" + getModuleDesc();
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (SystemUtil.getOsInfo().isLinux()) {
                Runtime.getRuntime().exec("systemctl restart " + setupProject.getProjectCode());
            } else if (SystemUtil.getOsInfo().isWindows()) {
                //todo
            }
        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            doException(request, busiDesc, model, e);
        }
        ret.put(KEY_RESULT_CODE, code);
        ret.put(KEY_RESULT_MSG, "项目运行状态更新成功！");
        return ret.toJSONString();
    }


    /**
     * 检测是否初始安装，是否已经安装了base-manager,如果没有则进入引导界面进行安装
     *
     * @return
     */
    @GetMapping("/check")
    @UnAuth
    public Rest<HashMap<String, Object>> check() {
        Boolean bool = false;
        HashMap<String, Object> data = new HashMap<>();
        data.put("resource", false);
        data.put("project", false);
        try {
            //todo 检测是否初始安装，是否已经安装了base-manager,如果没有则进入引导界面进行安装
            SetupProjectQuery setupProjectQuery = new SetupProjectQuery();
            setupProjectQuery.setProjectCode(基础服务后端.getValue());
            SetupProjectEntity setupProjectEntity = this.service.selectOne(setupProjectQuery);
            if (ObjectUtils.isEmpty(setupProjectEntity)) {
                throw new AppException("基础服务不存在");
            }
            //判断资源文件是否已经上传
            String projectPath = this.publishPath + "/temp/project/";
            boolean exist = FileUtil.exist(projectPath);
            if (exist) {
                data.put("resource", true);
            }
            if (setupProjectEntity.getProjectStatus() > ProjectStatusEnum.未部署.getValue()) {
                bool = true;
                data.put("project", bool);
                data.put("siteCode", setupProjectEntity.getSiteCode());
                return Rest.ok("项目工程已部署", data);
            }
            return Rest.ok("项目工程未部署，进入引导界面", data);
        } catch (Exception e) {
            log.error("检测异常", e);
            return Rest.fail(e.getMessage());
        }
    }


    public static void main(String[] args) {
        //System.out.println(FileUtil.isDirectory("E://pic"));
        System.out.println(Validator.isIpv4("127.0.0.1121"));

    }

}