package com.mortals.xhx.module.setup.web;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.ZipUtil;
import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.annotation.UnAuth;
import com.mortals.framework.exception.AppException;
import com.mortals.xhx.base.system.upload.service.UploadService;
import com.mortals.xhx.common.utils.EncodeUtil;
import com.mortals.xhx.module.setup.mode.ResourceSetupEntity;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.ObjectUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.nio.charset.Charset;

import static com.mortals.framework.web.BaseController.*;


@RestController
@RequestMapping("resource")
@Slf4j
public class SetupResourceController {

    @Value("${upload.path}")
    @Getter
    private String filePath;

    @Autowired
    private UploadService uploadService;

    @PostMapping("/init")
    @UnAuth
    public String initDb(@RequestBody ResourceSetupEntity resourceSetupEntity) {
        JSONObject ret = new JSONObject();
        int code = VALUE_RESULT_SUCCESS;
        try {
            if (ObjectUtils.isEmpty(resourceSetupEntity.getResourceFilePath())) {
                throw new AppException("请上传资源文件");
            }
            String targetFilePath = uploadService.getFilePath(resourceSetupEntity.getResourceFilePath());
            if (FileUtil.isEmpty(new File(targetFilePath))) {
                throw new AppException("部署文件不存在！");
            }

            if (!FileUtil.getSuffix(resourceSetupEntity.getResourceFilePath()).equals("zip")) {
                throw new AppException("部署只支持zip文件！");
            }

            if (!FileUtil.isDirectory(filePath)) {
                //部署路径是否存在 如果不存在 创建目录，
                FileUtil.mkdir(filePath);
            }

            String fileEncode = "UTF-8";
            try {
                fileEncode = EncodeUtil.getEncode(targetFilePath, true);
            } catch (Exception e) {
                log.error("异常", e);
            }
            ZipUtil.unzip(targetFilePath, filePath, Charset.forName(fileEncode));

            ret.put(KEY_RESULT_MSG, "初始化资源文件成功!");

        } catch (Exception e) {
            code = VALUE_RESULT_FAILURE;
            log.error("导入资源文件失败", e);
            ret.put(KEY_RESULT_MSG, e.getMessage());
        }
        ret.put(KEY_RESULT_CODE, code);
        return ret.toJSONString();
    }

    public static void main(String[] args) {

        System.out.println(FileUtil.isDirectory("E://pic"));

    }

}